/*==========================================================
 * arraySum.c - esempio per capire come funzionano i mex file
 *
 * Prendo due matrici 1xN in input (inMatrix1 e inMatrix2) 
 * e restituisco la somma in outputs in una matrice 1xN (outMatrix)
 *
 * Stavolta facciamo fare la somma a un kernel di Cuda
 *
 *========================================================*/
#include "/usr/local/MATLAB/R2012a/extern/include/mex.h"
#include "arraySum.h"

/* the gateway function */
/* nlhs = numero argomenti in output
/* nrhs = numero argomenti in input
/* prhs = array di puntatori ai dati di input (ha dimensione nrhs)
/* plhs = array di puntatori ai dati di output (ha dimensione nlhs)*/
void mexFunction(int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[]) {
	double *inMatrix1;               /* 1xN input matrix */
	double *inMatrix2;
    mwSize ncols;                   /* size of matrix */
    double *outMatrix;              /* output matrix */
	if(nrhs!=2) {
        mexErrMsgIdAndTxt("arraySum:nrhs","Two inputs required.");
    }
    if(nlhs!=1) {
        mexErrMsgIdAndTxt("arraySum:nlhs","One output required.");
    }
	
	/* check that number of rows of input argument is 1 */
    if(mxGetM(prhs[0])!=1) {
        mexErrMsgIdAndTxt("arraySum:notRowVector","Input must be a row vector.");
    }
	if(mxGetM(prhs[1])!=1) {
        mexErrMsgIdAndTxt("arraySum:notRowVector","Input must be a row vector.");
    }
	/* create a pointer to the real data in the input matrix  */
    inMatrix1 = mxGetPr(prhs[0]);
    inMatrix2 = mxGetPr(prhs[1]);
	
	/* check that the dimension of the input array as the same */
	if(mxGetN(prhs[0]) != mxGetN(prhs[1])) {
		mexErrMsgIdAndTxt("arraySum:notTheSameDimension","Input must have a same number of column.");
	}
	
	/* get dimensions of the input matrix */
    ncols = mxGetN(prhs[1]);
	
	   /* create the output matrix */
    plhs[0] = mxCreateDoubleMatrix(1,ncols,mxREAL);

    /* get a pointer to the real data in the output matrix */
    outMatrix = mxGetPr(plhs[0]);
	
	   /* call the cuda kernel */
    arraySum(inMatrix1,inMatrix2,outMatrix,ncols);
}

